/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.1a  (April 3, 1998).                                                *
*                                                                                    *
*  Definition of JPFlatMenuGadget class.                                             *
*                                                                                    *
*  Read JPFLTMGA.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/

#include <owl/pch.h>

#if !defined(OWL_UIHELPER_H)
#include <owl/uihelper.h>
#endif
#include "jpfltmga.h"

const int DefaultTextLen = 512;

//{{JPFlatMenuGadget Implementation}}

JPFlatMenuGadget::JPFlatMenuGadget(TMenu& menu, TWindow* window, int id, TBorderStyle borderstyle, const char far* text, TAlign align, uint numChars, TFont* font, bool Flat)
:
    TTextGadget(id, borderstyle, align, numChars, text, font),
    MenuTarget(window)
{
  // INSERT>> Your constructor code here.
  SetShrinkWrap(true, true);
  GadgetPopupMenu = new TPopupMenu(menu);
  DefaultBorderStyle = borderstyle;
  AutoTracking = Flat;

  char menuText[DefaultTextLen];
  if (!text)
    {
    if (GadgetPopupMenu->GetMenuString(0, menuText, DefaultTextLen, MF_BYPOSITION));
       SetText(menuText);
    }
  if ((AutoTracking) && (DefaultBorderStyle==ButtonUp))
       SetFlatState(fsNormal);
}


JPFlatMenuGadget::~JPFlatMenuGadget()
{
  // INSERT>> Your destructor code here.
  delete GadgetPopupMenu;
}


void JPFlatMenuGadget::LButtonDown(uint modKeys, TPoint& point)
{
  TTextGadget::LButtonDown(modKeys, point);

  // INSERT>> Your code here.
   TWindow* parent= Window->GetParentO();
   TDecoratedFrame* frame= parent ? TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
   while (parent && !frame)
   {
     parent = parent->GetParentO();
     if (parent)
       frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
   }
   if (frame)
   	ParentWnd = *frame;
   else ParentWnd = *MenuTarget;
   if (ParentWnd)
     {
      TTextGadget::MouseLeave(0,point);
      TRect rect = GetBounds();
      TPoint p2(rect.TopLeft());
      Window->ClientToScreen(p2);
      rect.right = p2.x + (rect.right - rect.left);
      rect.bottom = p2.y + (rect.bottom - rect.top);
      rect.left = p2.x;
      rect.top = p2.y;
      p2.y = rect.bottom;
      if (DefaultBorderStyle==ButtonUp)
         SetBorderStyle(ButtonDn);
      #if defined(BI_PLAT_WIN16)
      GadgetPopupMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, p2, 0, ParentWnd);
      #elif defined(BI_PLAT_WIN32)
      TrackPopupMenuEx(TPM_VERTICAL, p2, ParentWnd, &rect);
      #endif
      if (GetBorderStyle()!=DefaultBorderStyle)
         SetBorderStyle(DefaultBorderStyle);
      }
}


void JPFlatMenuGadget::MouseEnter(uint modKeys, TPoint& point)
{
  TTextGadget::MouseEnter(modKeys, point);

  // INSERT>> Your code here.
  if (Window->GetHintMode() == TGadgetWindow::EnterHints)
        Window->SetHintCommand(GetId());
  if ((AutoTracking) && (GetEnabled()) && (DefaultBorderStyle==ButtonUp))
  		SetFlatState(fsUp);
}


void JPFlatMenuGadget::MouseLeave(uint modKeys, TPoint& point)
{
  TTextGadget::MouseLeave(modKeys, point);

  // INSERT>> Your code here.
  if (Window->GetHintMode() == TGadgetWindow::EnterHints)
        Window->SetHintCommand(-1);
  if ((AutoTracking) && (DefaultBorderStyle==ButtonUp))
        SetFlatState(fsNormal);
}

void
JPFlatMenuGadget::SetFlatState(FlatState state, bool update)
{
switch (state)
	{
   case fsNormal:
      CurrentFlatState = fsNormal;
      break;

   case fsUp:
      if (GetEnabled())
      CurrentFlatState = fsUp;
      else return;
      break;

   case fsDown:
      if (GetEnabled())
      CurrentFlatState = fsDown;
      else return;
      break;
   }
if (update)
	{
   InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
	Update();
   }
}

JPFlatMenuGadget::FlatState
JPFlatMenuGadget::GetFlatState() const
{
  return  CurrentFlatState;
}

void JPFlatMenuGadget::PaintBorder(TDC& dc)
{
   if ((AutoTracking) && (DefaultBorderStyle==ButtonUp))
   	{
      if (GetFlatState()==fsUp)
         {
         TRect rect(0,0,Bounds.Width(),Bounds.Height());
         #if defined(BI_PLAT_WIN16)
         TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::WndRaised), TUIBorder::Rect | TUIBorder::Adjust);
         #elif defined(BI_PLAT_WIN32)
         TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::RaisedInner), TUIBorder::Rect | TUIBorder::Adjust);
         #endif
         border.Paint(dc);
         }
      else if (GetFlatState()==fsDown)
         {
         BorderStyle = Recessed;
         TTextGadget::PaintBorder(dc);
         }
      }
   else  TTextGadget::PaintBorder(dc);
}

void JPFlatMenuGadget::SetTracking(bool enable)
{
  if (enable)
  	{
   AutoTracking = true;
   SetFlatState(fsNormal);
   }
  else
  	{
   AutoTracking = false;
   SetBorderStyle(DefaultBorderStyle);
   InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
   Update();
   }
}

void
JPFlatMenuGadget::SetBorderStyle(TBorderStyle bs)
{
if ((AutoTracking) && (DefaultBorderStyle==ButtonUp))
	{
   if (bs==ButtonDn)
   	SetFlatState(fsDown);
   else if (bs==ButtonUp)
      SetFlatState(fsNormal);
   }
else
   {
	TTextGadget::SetBorderStyle(bs);
   Invalidate();
	Update();
   }
}

#if defined(BI_PLAT_WIN32)
bool JPFlatMenuGadget::TrackPopupMenuEx(uint flags, const TPoint& point,
                                        HWND wnd, const TRect* rect)
{
  TPMPARAMS tpm;

  PRECONDITION(GadgetPopupMenu->GetHandle());
  tpm.cbSize=sizeof(TPMPARAMS);
  tpm.rcExclude=*rect;
  return ::TrackPopupMenuEx(GadgetPopupMenu->GetHandle(), flags, point.x, point.y, wnd, &tpm);
}
#endif

